require 'ruby_man/GameObjects/moving_object'
require 'ruby_man/GameObjects/ghost_corpse'

module RubyMan
  # Ghost that chases player
  class Ghost < MovingObject
    def created
      self.sprite = Sprite['ghost_left']
      @spawn_x, @spawn_y = @grid_x, @grid_y
      stop_scared
    end

    def update(delta)
      super
      update_scared(delta) unless @dead
    end

    def grid_step
      select_direction!
    end

    def possible_directions
      directions = []
      Direction.each do |dir|
        next unless dir != @direction.opposite && direction_free(dir)
        directions.push(dir)
      end
      directions
    end

    def scared_directions
      to_pl = directions_to_player
      possible = possible_directions
      away = possible - to_pl
      return possible if away.empty?
      away
    end

    def chase_directions
      directions = possible_directions
      directions_to_player.each do |dir|
        2.times { directions.push(dir) } if directions.include?(dir)
      end
      directions
    end

    def weighted_directions
      directions = @scared ? scared_directions : chase_directions
      directions.push(Direction.none) if directions.empty?
      directions
    end

    def directions_to_player
      return [] unless @game.player
      rel_x = @game.player.grid_x - @grid_x
      rel_y = @game.player.grid_y - @grid_y
      directions = []
      directions.push(Direction.right) if rel_x > 0
      directions.push(Direction.left) if rel_x < 0
      directions.push(Direction.down) if rel_y > 0
      directions.push(Direction.up) if rel_y < 0
      directions
    end

    def select_direction!
      directions = weighted_directions
      change_direction(directions[rand(directions.size)])
    end

    def change_direction(direction)
      @direction = direction
      return if @scared
      case direction
      when Direction.left then self.sprite = Sprite['ghost_left']
      when Direction.right then self.sprite = Sprite['ghost_right']
      when Direction.up then self.sprite = Sprite['ghost_up']
      when Direction.down then self.sprite = Sprite['ghost_down']
      end
    end

    def set_pos(x, y)
      super
      select_direction! if @game
    end

    def scare
      @scared = 7.0
      @speed = 1.5
      update_scared(0.0)
    end

    def scared?
      @scared != nil
    end

    def update_scared(delta)
      return unless @scared
      @scared -= delta
      return stop_scared if @scared <= 0.0
      if @scared > 2.0
        self.sprite = Sprite['ghost_scared']
      else
        self.sprite = Sprite['ghost_stop_scared']
      end
    end

    def stop_scared
      @speed = 2.5
      @scared = nil
      # reset sprite according to current direction
      change_direction(@direction)
    end

    def die
      ResourceManager['sfx_eatghost'].play
      @game.score += 500
      @game.add_object(GhostCorpse.new(@grid_x, @grid_y,
                                       @phase, @direction,
                                       @spawn_x, @spawn_y))
      destroy
    end
  end
end
