require 'ruby_man/GameObjects/game_object'
require 'ruby_man/aabb'

module RubyMan
  # GameObject aligned to game's grid
  class GridObject < GameObject
    attr_accessor :grid_x, :grid_y, :depth
    attr_reader :sprite
    attr_accessor :image_index, :image_speed, :image_angle
    attr_accessor :solid

    def initialize
      super
      @grid_x = -1
      @grid_y = -1
      @depth = 0
      @image_index = 0
      @image_speed = 1
      @image_angle = 0
      @solid = false
      @scale = 1.0
    end

    def update(delta)
      super
      return unless sprite
      @image_index += @image_speed * delta * @sprite.frames.size
      @image_index %= @sprite.frames.size
    end

    def draw
      super
      return unless sprite
      sprite.draw(pixel_x, pixel_y, depth,
                  image_angle, image_index.to_i, @scale)
    end

    def sprite=(new_sprite)
      return if @sprite == new_sprite
      @image_index = 0
      @sprite = new_sprite
    end

    def pixel_x
      @grid_x * 32
    end

    def pixel_y
      @grid_y * 32
    end

    def bbox
      AABB.new(pixel_x, pixel_y,
               pixel_x + @sprite.width, pixel_y + @sprite.height)
    end

    def distance_to(other)
      rel_x = other.pixel_x - pixel_x
      rel_y = other.pixel_y - pixel_y
      Math.sqrt(rel_x.abs2 + rel_y.abs2)
    end

    def set_pos(x, y)
      @grid_x = x
      @grid_y = y
      @phase = 0
    end
  end
end
