require 'ruby_man/resource_manager'
require 'ruby_man/GameObjects/moving_object'

module RubyMan
  # RubyMan object for decorating main menu
  class MenuRubyMan < MovingObject
    def created
      self.sprite = Sprite['rubyman']
      set_pos(-4, 8)
      change_direction(Direction.right)
      @speed = 3
      @image_speed = 3
    end

    def grid_step
      if @direction == Direction.right
        change_direction(Direction.left) if @grid_x >= 31
      else
        change_direction(Direction.right) if @grid_x <= -4
      end
    end
  end

  # Ghost object for decorating main menu
  class MenuGhost < MovingObject
    def created
      set_pos(-6, 8)
      @speed = 3
      @image_speed = 3
      set_chase
    end

    def set_chase
      @direction = Direction.right
      self.sprite = Sprite['ghost_right']
    end

    def set_run
      @direction = Direction.left
      self.sprite = Sprite['ghost_scared']
    end

    def grid_step
      if @direction == Direction.right
        set_run if @grid_x >= 29
      else
        set_chase if @grid_x <= -6
      end
    end
  end

  # title screen showing high scores
  class Menu < GameObject
    def created
      @logo = Sprite['logo']
      @text = ResourceManager['text_normal']
      @large_text = ResourceManager['text_large']
      @game.add_object(MenuRubyMan.new)
      @game.add_object(MenuGhost.new)
    end

    def button_down(id)
      @game.start if id == Gosu::KbEnter || id == Gosu::KbReturn
    end

    def draw
      @logo.draw_centered(@game.x_res / 2, 92)
      @large_text.draw_centered('Press Enter to start!', @game.x_res / 2, 200)
      draw_scores(@game.x_res / 2 - 192, @game.y_res / 2)
    end

    def draw_scores(x, y)
      y_offset = 0
      @game.scores.each do |entry|
        @text.draw(entry[0], x, y + y_offset, 0)
        @text.draw(entry[1], x + 128, y + y_offset, 0)
        @text.draw(entry[2], x + 320, y + y_offset, 0)
        y_offset += 32
      end
    end
  end
end
