require 'gosu'
require 'ruby_man/GameObjects/moving_object'
require 'ruby_man/GameObjects/food'
require 'ruby_man/GameObjects/ghost'
require 'ruby_man/GameObjects/door'

module RubyMan
  # player (RubyMan) object
  class Player < MovingObject
    def created
      self.sprite = Sprite['rubyman']
      @image_speed = 3
      @next_direction = Direction.none
      @speed = 3
      @is_dead = false
      @game.player = self
    end

    def button_down(id)
      case id
      when Gosu::KbLeft then @next_direction = Direction.left
      when Gosu::KbUp then @next_direction = Direction.up
      when Gosu::KbRight then @next_direction = Direction.right
      when Gosu::KbDown then @next_direction = Direction.down
      end
    end

    def update(delta)
      dead if @is_dead
      super
    end

    def grid_step
      eat_food
      change_direction(@next_direction)
      @direction = Direction.none unless direction_free(@direction)
    end

    def direction_free(direction)
      x = @grid_x + direction.rel_x
      y = @grid_y + direction.rel_y
      @game.objects_at(x, y).each do |obj|
        return false if obj.solid || obj.is_a?(Door)
      end
      true
    end

    def collision(obj)
      return unless obj.is_a?(Ghost)
      return unless distance_to(obj) <= 16
      if obj.scared?
        obj.die
      else
        die
      end
    end

    def die
      ResourceManager['sfx_death'].play
      @game.pause(2000)
      @is_dead = true
    end

    def dead
      if @game.lives > 0
        @game.lives -= 1
        @game.restart
      else
        @game.game_over
      end
    end

    def eat_food
      @game.objects_at(@grid_x, @grid_y).each do |obj|
        next unless obj.is_a?(Food)
        obj.destroy
        @game.score += 10
      end
    end

    def change_direction(direction)
      dir_x = @grid_x + direction.rel_x
      dir_y = @grid_y + direction.rel_y
      @game.objects_at(dir_x, dir_y).each do |obj|
        return if obj.solid || obj.is_a?(Door)
      end
      super
    end
  end
end
