module RubyMan
  # Class representing left/right/up/down/none direction in 2D space
  class Direction
    def initialize(dir = :none)
      @direction = dir
    end

    def self.none
      Direction.new
    end

    def self.left
      Direction.new(:left)
    end

    def self.right
      Direction.new(:right)
    end

    def self.up
      Direction.new(:up)
    end

    def self.down
      Direction.new(:down)
    end

    def self.each
      return to_enum unless block_given?
      yield Direction.right
      yield Direction.down
      yield Direction.left
      yield Direction.up
    end

    def to_i
      case @direction
      when :right
        return 0
      when :down
        return 1
      when :left
        return 2
      when :up
        return 3
      end
    end

    def self.from_i(num)
      case num % 4
      when 0
        return Direction.right
      when 1
        return Direction.down
      when 2
        return Direction.left
      when 3
        return Direction.up
      end
    end

    def rel_x
      i = to_i
      return 0 unless i
      (1 - i) * ((i + 1) % 2)
    end

    def rel_y
      i = to_i
      return 0 unless i
      (2 - i) * (i % 2)
    end

    def opposite
      i = to_i
      return Direction.none unless i
      Direction.from_i(to_i + 2)
    end

    def inverted
      Direction.each.select { |dir| dir != self }
    end

    def to_angle
      i = to_i
      return 0 unless i
      i * 90
    end

    def ==(other)
      to_i == other.to_i
    end

    def eql?(other)
      self == other
    end

    def hash
      @direction.hash
    end
  end
end
