require 'ruby_man/GameObjects/wall'
require 'ruby_man/GameObjects/player'
require 'ruby_man/GameObjects/food'
require 'ruby_man/GameObjects/ghost'
require 'ruby_man/GameObjects/hud'
require 'ruby_man/GameObjects/door'
require 'ruby_man/GameObjects/super_food'

module RubyMan
  # mixin for loading levels from files
  # required method - add_object, level_loaded
  module LevelLoader
    def load_level_string(level_string, restart)
      y = 0
      level_string.lines do |line|
        x = -1
        line.each_char do |char|
          x += 1
          obj = LevelLoader.object_from_char(char)
          next unless obj
          next if restart && obj.class.persistent == true
          obj.set_pos(x, y)
          add_object(obj)
        end
        y += 1
      end

      add_object(Hud.new)

      level_loaded
    end

    def load_level_file(level_name, restart)
      f = File.open(ResourceManager.media(level_name))
      load_level_string(f.read, restart)
    end

    def self.object_from_char(char)
      case char.downcase
      when 'p' then return Player.new
      when '.' then return Food.new
      when ':' then return SuperFood.new
      when 'g' then return Ghost.new
      when '_' then return Door.new
      end
      Wall.from_char(char) unless Wall.chars.index(char).nil?
    end
  end
end
